/*
 * Author: Kipp Cannon <kipp@gravity.phys.uwm.edu>
 * Revision: $Id: FrequencySeries.m4,v 1.1 2005/01/28 23:38:05 kipp Exp $
 */

#include <string.h>
#include <lal/Date.h>
#include <lal/LALDatatypes.h>
#include <lal/LALStdlib.h>
#include <lal/LALStatusMacros.h>
#include <lal/LALErrno.h>
#include <lal/LALError.h>
#include <lal/Sequence.h>
#include <lal/FrequencySeries.h>

NRCSID(FREQUENCYSERIESC, "$Id: FrequencySeries.m4,v 1.1 2005/01/28 23:38:05 kipp Exp $");




void XLALDestroyREAL4FrequencySeries (
	REAL4FrequencySeries *series
)
{
	if(series)
		XLALDestroyREAL4Sequence (series->data);
	LALFree(series);
}


void LALDestroyREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyREAL4FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyREAL4FrequencySeries (series);
	RETURN(status);
}


REAL4FrequencySeries *XLALCreateREAL4FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	REAL4FrequencySeries *new;
	REAL4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateREAL4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateREAL4FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateREAL4FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


REAL4FrequencySeries *XLALCutREAL4FrequencySeries (
	REAL4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	REAL4FrequencySeries *new;
	REAL4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutREAL4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries **output,
	REAL4FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutREAL4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutREAL4FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkREAL4FrequencySeries (
	REAL4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkREAL4Sequence (series->data, first, length));
}


void LALShrinkREAL4FrequencySeries (
	LALStatus *status,
	REAL4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkREAL4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkREAL4FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyREAL8FrequencySeries (
	REAL8FrequencySeries *series
)
{
	if(series)
		XLALDestroyREAL8Sequence (series->data);
	LALFree(series);
}


void LALDestroyREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyREAL8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyREAL8FrequencySeries (series);
	RETURN(status);
}


REAL8FrequencySeries *XLALCreateREAL8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	REAL8FrequencySeries *new;
	REAL8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateREAL8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateREAL8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateREAL8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


REAL8FrequencySeries *XLALCutREAL8FrequencySeries (
	REAL8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	REAL8FrequencySeries *new;
	REAL8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutREAL8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries **output,
	REAL8FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutREAL8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutREAL8FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkREAL8FrequencySeries (
	REAL8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkREAL8Sequence (series->data, first, length));
}


void LALShrinkREAL8FrequencySeries (
	LALStatus *status,
	REAL8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkREAL8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkREAL8FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX8Sequence (series->data);
	LALFree(series);
}


void LALDestroyCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyCOMPLEX8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyCOMPLEX8FrequencySeries (series);
	RETURN(status);
}


COMPLEX8FrequencySeries *XLALCreateCOMPLEX8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	COMPLEX8FrequencySeries *new;
	COMPLEX8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateCOMPLEX8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateCOMPLEX8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


COMPLEX8FrequencySeries *XLALCutCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	COMPLEX8FrequencySeries *new;
	COMPLEX8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries **output,
	COMPLEX8FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutCOMPLEX8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutCOMPLEX8FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkCOMPLEX8FrequencySeries (
	COMPLEX8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkCOMPLEX8Sequence (series->data, first, length));
}


void LALShrinkCOMPLEX8FrequencySeries (
	LALStatus *status,
	COMPLEX8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkCOMPLEX8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkCOMPLEX8FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX16Sequence (series->data);
	LALFree(series);
}


void LALDestroyCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyCOMPLEX16FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyCOMPLEX16FrequencySeries (series);
	RETURN(status);
}


COMPLEX16FrequencySeries *XLALCreateCOMPLEX16FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	COMPLEX16FrequencySeries *new;
	COMPLEX16Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX16Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateCOMPLEX16FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateCOMPLEX16FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


COMPLEX16FrequencySeries *XLALCutCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series,
	size_t first,
	size_t length
)
{
	COMPLEX16FrequencySeries *new;
	COMPLEX16Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX16Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries **output,
	COMPLEX16FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutCOMPLEX16FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutCOMPLEX16FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkCOMPLEX16FrequencySeries (
	COMPLEX16FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkCOMPLEX16Sequence (series->data, first, length));
}


void LALShrinkCOMPLEX16FrequencySeries (
	LALStatus *status,
	COMPLEX16FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkCOMPLEX16FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkCOMPLEX16FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyINT2FrequencySeries (
	INT2FrequencySeries *series
)
{
	if(series)
		XLALDestroyINT2Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyINT2FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyINT2FrequencySeries (series);
	RETURN(status);
}


INT2FrequencySeries *XLALCreateINT2FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	INT2FrequencySeries *new;
	INT2Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT2Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT2Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT2FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT2FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


INT2FrequencySeries *XLALCutINT2FrequencySeries (
	INT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	INT2FrequencySeries *new;
	INT2Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT2Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries **output,
	INT2FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutINT2FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutINT2FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkINT2FrequencySeries (
	INT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkINT2Sequence (series->data, first, length));
}


void LALShrinkINT2FrequencySeries (
	LALStatus *status,
	INT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT2FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT2FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyUINT2FrequencySeries (
	UINT2FrequencySeries *series
)
{
	if(series)
		XLALDestroyUINT2Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT2FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyUINT2FrequencySeries (series);
	RETURN(status);
}


UINT2FrequencySeries *XLALCreateUINT2FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	UINT2FrequencySeries *new;
	UINT2Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT2Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT2FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT2FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


UINT2FrequencySeries *XLALCutUINT2FrequencySeries (
	UINT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	UINT2FrequencySeries *new;
	UINT2Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries **output,
	UINT2FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutUINT2FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutUINT2FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkUINT2FrequencySeries (
	UINT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkUINT2Sequence (series->data, first, length));
}


void LALShrinkUINT2FrequencySeries (
	LALStatus *status,
	UINT2FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT2FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT2FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyINT4FrequencySeries (
	INT4FrequencySeries *series
)
{
	if(series)
		XLALDestroyINT4Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyINT4FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyINT4FrequencySeries (series);
	RETURN(status);
}


INT4FrequencySeries *XLALCreateINT4FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	INT4FrequencySeries *new;
	INT4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT4Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT4FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT4FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


INT4FrequencySeries *XLALCutINT4FrequencySeries (
	INT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	INT4FrequencySeries *new;
	INT4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT4Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries **output,
	INT4FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutINT4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutINT4FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkINT4FrequencySeries (
	INT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkINT4Sequence (series->data, first, length));
}


void LALShrinkINT4FrequencySeries (
	LALStatus *status,
	INT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT4FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyUINT4FrequencySeries (
	UINT4FrequencySeries *series
)
{
	if(series)
		XLALDestroyUINT4Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT4FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyUINT4FrequencySeries (series);
	RETURN(status);
}


UINT4FrequencySeries *XLALCreateUINT4FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	UINT4FrequencySeries *new;
	UINT4Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT4Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT4FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT4FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


UINT4FrequencySeries *XLALCutUINT4FrequencySeries (
	UINT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	UINT4FrequencySeries *new;
	UINT4Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries **output,
	UINT4FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutUINT4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutUINT4FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkUINT4FrequencySeries (
	UINT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkUINT4Sequence (series->data, first, length));
}


void LALShrinkUINT4FrequencySeries (
	LALStatus *status,
	UINT4FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT4FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT4FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyINT8FrequencySeries (
	INT8FrequencySeries *series
)
{
	if(series)
		XLALDestroyINT8Sequence (series->data);
	LALFree(series);
}


void LALDestroyINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyINT8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyINT8FrequencySeries (series);
	RETURN(status);
}


INT8FrequencySeries *XLALCreateINT8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	INT8FrequencySeries *new;
	INT8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateINT8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT8Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateINT8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateINT8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


INT8FrequencySeries *XLALCutINT8FrequencySeries (
	INT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	INT8FrequencySeries *new;
	INT8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyINT8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries **output,
	INT8FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutINT8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutINT8FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkINT8FrequencySeries (
	INT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkINT8Sequence (series->data, first, length));
}


void LALShrinkINT8FrequencySeries (
	LALStatus *status,
	INT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkINT8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkINT8FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}





void XLALDestroyUINT8FrequencySeries (
	UINT8FrequencySeries *series
)
{
	if(series)
		XLALDestroyUINT8Sequence (series->data);
	LALFree(series);
}


void LALDestroyUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries *series
)
{
	INITSTATUS(status, "LALDestroyUINT8FrequencySeries", FREQUENCYSERIESC);
	XLALDestroyUINT8FrequencySeries (series);
	RETURN(status);
}


UINT8FrequencySeries *XLALCreateUINT8FrequencySeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaF,
	const LALUnit *sampleUnits,
	size_t length
)
{
	UINT8FrequencySeries *new;
	UINT8Sequence *sequence;

	new = LALMalloc(sizeof(*new));
	sequence = XLALCreateUINT8Sequence (length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		return(NULL);
	}

	if(new->name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaF = deltaF;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return(new);
}


void LALCreateUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries **output,
	const CHAR *name,
	LIGOTimeGPS epoch,
	REAL8 f0,
	REAL8 deltaF,
	LALUnit sampleUnits,
	size_t length
)
{
	INITSTATUS(status, "LALCreateUINT8FrequencySeries", FREQUENCYSERIESC);
	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	*output = XLALCreateUINT8FrequencySeries (name, &epoch, f0, deltaF, &sampleUnits, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);
	RETURN(status);
}


UINT8FrequencySeries *XLALCutUINT8FrequencySeries (
	UINT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	UINT8FrequencySeries *new;
	UINT8Sequence *sequence;

	if(!series || !series->data)
		return(NULL);

	new = LALMalloc(sizeof(*new));
	sequence = XLALCutUINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		LALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		return(NULL);
	}

	*new = *series;
	new->data = sequence;
	new->f0 += first * new->deltaF;

	return(new);
}


void LALCutUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries **output,
	UINT8FrequencySeries *input,
	size_t first,
	size_t length
)
{
	INITSTATUS(status, "LALCutUINT8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(output != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(input->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= input->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	*output = XLALCutUINT8FrequencySeries (input, first, length);
	ASSERT(*output != NULL, status, LAL_NOMEM_ERR, LAL_NOMEM_MSG);

	RETURN(status);
}


size_t XLALShrinkUINT8FrequencySeries (
	UINT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	if(!series)
		return(0);

	series->f0 += first * series->deltaF;
	return(XLALShrinkUINT8Sequence (series->data, first, length));
}


void LALShrinkUINT8FrequencySeries (
	LALStatus *status,
	UINT8FrequencySeries *series,
	size_t first,
	size_t length
)
{
	size_t new_length;

	INITSTATUS(status, "LALShrinkUINT8FrequencySeries", FREQUENCYSERIESC);

	ASSERT(series != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(series->data != NULL, status, LAL_NULL_ERR, LAL_NULL_MSG);
	ASSERT(first + length <= series->data->length, status, LAL_RANGE_ERR, LAL_RANGE_MSG);
	new_length = XLALShrinkUINT8FrequencySeries (series, first, length);
	ASSERT(new_length == length, status, LAL_FAIL_ERR, LAL_FAIL_MSG);
	RETURN(status);
}

